@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; data object ref/set stuff
 
@@ -27,7 +27,7 @@
   (:info name offset lowtag)
   (:results)
   (:generator 1
-    (cond ((emit-gc-barrier-store-p name)
+    (cond ((emit-code-page-write-barrier-p name)
            (inst push (encode-value-if-immediate value))
            (inst push offset)
            (inst push object)
@@ -78,7 +78,7 @@
          (old :scs (descriptor-reg any-reg) :target eax)
          (new :scs (descriptor-reg any-reg)))
   (:temporary (:sc descriptor-reg :offset eax-offset) eax)
-  #!+sb-thread
+  #+sb-thread
   (:temporary (:sc descriptor-reg) tls)
   (:results (result :scs (descriptor-reg any-reg)))
   (:policy :fast-safe)
@@ -90,7 +90,7 @@
     (let ((unbound (generate-error-code vop 'unbound-symbol-error symbol))
           (check (gen-label)))
       (move eax old)
-      #!+sb-thread
+      #+sb-thread
       (progn
         (loadw tls symbol symbol-tls-index-slot other-pointer-lowtag)
         ;; Thread-local area, no LOCK needed.
@@ -129,7 +129,7 @@
       (inst cmp value unbound-marker-widetag)
       (inst jmp :e err-lab))))
 
-#!+sb-thread
+#+sb-thread
 (progn
   (define-vop (set)
     (:args (symbol :scs (descriptor-reg))
@@ -190,7 +190,7 @@
         (loadw value object symbol-value-slot other-pointer-lowtag)
         (emit-label ret-lab)))))
 
-#!-sb-thread
+#-sb-thread
 (progn
   (define-vop (symbol-value symbol-global-value)
     (:translate symeval))
@@ -198,7 +198,7 @@
     (:translate symeval))
   (define-vop (set %set-symbol-global-value)))
 
-#!+sb-thread
+#+sb-thread
 (define-vop (boundp)
   (:translate boundp)
   (:policy :fast-safe)
@@ -216,14 +216,14 @@
       (emit-label check-unbound-label)
       (inst cmp value unbound-marker-widetag))))
 
-#!-sb-thread
+#-sb-thread
 (define-vop (boundp)
   (:translate boundp)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg)))
   (:conditional :ne)
   (:generator 9
-    (inst cmp (make-ea-for-object-slot object symbol-value-slot
+    (inst cmp (object-slot-ea object symbol-value-slot
                                        other-pointer-lowtag)
           unbound-marker-widetag)))
 
@@ -236,7 +236,7 @@
   (:result-types positive-fixnum)
   (:generator 2
     ;; The symbol-hash slot of NIL holds NIL because it is also the
-    ;; cdr slot, so we have to strip off the two low bits to make sure
+    ;; car slot, so we have to strip off the two low bits to make sure
     ;; it is a fixnum.  The lowtag selection magic that is required to
     ;; ensure this is explained in the comment in objdef.lisp
     (loadw res symbol symbol-hash-slot other-pointer-lowtag)
@@ -268,7 +268,7 @@
   (:temporary (:sc unsigned-reg) raw)
   (:results (result :scs (descriptor-reg)))
   (:generator 38
-    (inst lea raw (make-fixup 'closure-tramp :assembly-routine))
+    (inst mov raw (make-fixup 'closure-tramp :assembly-routine))
     (inst cmp (make-ea :byte :base function :disp (- fun-pointer-lowtag))
           simple-fun-widetag)
     (inst cmov :e raw
@@ -296,14 +296,14 @@
 ;;; symbol.
 ;;; See the "Chapter 9: Specials" of the SBCL Internals Manual.
 ;;
-#!+sb-thread
+#+sb-thread
 (macrolet
     ((def (vopname symbol-arg info sc-offset load-tls-index)
        `(define-vop (,vopname)
           (:args (val :scs (any-reg descriptor-reg)) ,@symbol-arg)
           ,@info
           (:temporary (:sc unsigned-reg ,@sc-offset) tls-index)
-          (:temporary (:sc unsigned-reg) bsp #!+win32 temp)
+          (:temporary (:sc unsigned-reg) bsp #+win32 temp)
           (:generator 10 ; cost is irrelevant. we explicitly pick a vop
            (load-binding-stack-pointer bsp)
            ,@load-tls-index
@@ -311,12 +311,12 @@
            (store-binding-stack-pointer bsp)
      ;; with-tls-ea on win32 causes tls-index to be an absolute address
      ;; which is problematic when UNBIND uses with-tls-ea too.
-           #!+win32(move temp tls-index)
+           #+win32(move temp tls-index)
            (with-tls-ea (EA :base tls-index :base-already-live-p t)
             (inst push EA :maybe-fs)
             (popw bsp (- binding-value-slot binding-size))
-            (storew #!-win32 tls-index
-                    #!+win32 temp
+            (storew #-win32 tls-index
+                    #+win32 temp
                     bsp (- binding-symbol-slot binding-size))
             (inst mov EA val :maybe-fs))))))
   (def bind ; bind a known symbol
@@ -331,7 +331,7 @@
         (inst call (make-fixup 'alloc-tls-index :assembly-routine))
         TLS-INDEX-VALID)))
 
-#!-sb-thread
+#-sb-thread
 (define-vop (dynbind)
   (:args (val :scs (any-reg descriptor-reg))
          (symbol :scs (descriptor-reg)))
@@ -345,7 +345,7 @@
     (storew symbol bsp (- binding-symbol-slot binding-size))
     (storew val symbol symbol-value-slot other-pointer-lowtag)))
 
-#!+sb-thread
+#+sb-thread
 (define-vop (unbind)
   (:temporary (:sc unsigned-reg) temp bsp tls-index)
   (:generator 0
@@ -362,7 +362,7 @@
     (storew 0 bsp binding-value-slot)
     (store-binding-stack-pointer bsp)))
 
-#!-sb-thread
+#-sb-thread
 (define-vop (unbind)
   (:temporary (:sc unsigned-reg) symbol value bsp)
   (:generator 0
@@ -393,8 +393,8 @@
     (inst cmp symbol unbound-marker-widetag)
     (inst jmp :eq skip)
     (loadw value bsp binding-value-slot)
-    #!-sb-thread (storew value symbol symbol-value-slot other-pointer-lowtag)
-    #!+sb-thread (with-tls-ea (EA :base symbol :base-already-live-p t)
+    #-sb-thread (storew value symbol symbol-value-slot other-pointer-lowtag)
+    #+sb-thread (with-tls-ea (EA :base symbol :base-already-live-p t)
                    (inst mov EA value :maybe-fs))
     (storew 0 bsp binding-symbol-slot)
 
@@ -458,7 +458,8 @@
   (:result-types positive-fixnum)
   (:generator 4
     (loadw res struct 0 instance-pointer-lowtag)
-    (inst shr res n-widetag-bits)))
+    (inst shr res n-widetag-bits)
+    (inst and res short-header-max-words))) ; clear special GC bit
 
 (define-full-reffer instance-index-ref *
   instance-slots-offset instance-pointer-lowtag
@@ -501,13 +502,13 @@
 
 ;;;; raw instance slot accessors
 
-(defun make-ea-for-raw-slot (object index &optional (displacement 0))
+(defun instance-slot-ea (object index &optional (displacement 0))
   ;; instance-init vops pass a literal integer, ref/set can use an immediate tn
   (let ((imm-index (cond ((integerp index) index)
                          ((sc-is index immediate) (tn-value index)))))
     (make-ea :dword :base object
              ;; If index is a register, it needs no scaling - it has tag bits.
-             :index (unless imm-index index) :scale 1
+             :index (unless imm-index index)
              :disp (- (ash (+ (or imm-index 0) displacement instance-slots-offset)
                            word-shift) instance-pointer-lowtag))))
 
@@ -519,7 +520,7 @@
   (:results (value :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:generator 5
-    (inst mov value (make-ea-for-raw-slot object index))))
+    (inst mov value (instance-slot-ea object index))))
 
 (define-vop (raw-instance-set/word)
   (:translate %raw-instance-set/word)
@@ -531,7 +532,7 @@
   (:results (result :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:generator 5
-    (inst mov (make-ea-for-raw-slot object index) value)
+    (inst mov (instance-slot-ea object index) value)
     (move result value)))
 
 (define-vop (raw-instance-init/word)
@@ -540,7 +541,7 @@
   (:arg-types * unsigned-num)
   (:info index)
   (:generator 5
-    (inst mov (make-ea-for-raw-slot object index) value)))
+    (inst mov (instance-slot-ea object index) value)))
 
 (define-vop (raw-instance-ref/signed-word)
   (:translate %raw-instance-ref/signed-word)
@@ -550,7 +551,7 @@
   (:results (value :scs (signed-reg)))
   (:result-types signed-num)
   (:generator 5
-    (inst mov value (make-ea-for-raw-slot object index))))
+    (inst mov value (instance-slot-ea object index))))
 
 (define-vop (raw-instance-set/signed-word)
   (:translate %raw-instance-set/signed-word)
@@ -562,7 +563,7 @@
   (:results (result :scs (signed-reg)))
   (:result-types signed-num)
   (:generator 5
-    (inst mov (make-ea-for-raw-slot object index) value)
+    (inst mov (instance-slot-ea object index) value)
     (move result value)))
 
 (define-vop (raw-instance-init/signed-word)
@@ -571,7 +572,7 @@
   (:arg-types * signed-num)
   (:info index)
   (:generator 5
-    (inst mov (make-ea-for-raw-slot object index) value)))
+    (inst mov (instance-slot-ea object index) value)))
 
 (define-vop (raw-instance-atomic-incf/word)
   (:translate %raw-instance-atomic-incf/word)
@@ -583,7 +584,7 @@
   (:results (result :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:generator 5
-    (inst xadd (make-ea-for-raw-slot object index) diff :lock)
+    (inst xadd (instance-slot-ea object index) diff :lock)
     (move result diff)))
 
 (define-vop (raw-instance-ref/single)
@@ -595,7 +596,7 @@
   (:result-types single-float)
   (:generator 5
     (with-empty-tn@fp-top(value)
-      (inst fld (make-ea-for-raw-slot object index)))))
+      (inst fld (instance-slot-ea object index)))))
 
 (define-vop (raw-instance-set/single)
   (:translate %raw-instance-set/single)
@@ -609,7 +610,7 @@
   (:generator 5
     (unless (zerop (tn-offset value))
       (inst fxch value))
-    (inst fst (make-ea-for-raw-slot object index))
+    (inst fst (instance-slot-ea object index))
     (cond
       ((zerop (tn-offset value))
         (unless (zerop (tn-offset result))
@@ -628,7 +629,7 @@
   (:info index)
   (:generator 5
     (with-tn@fp-top (value)
-      (inst fst (make-ea-for-raw-slot object index)))))
+      (inst fst (instance-slot-ea object index)))))
 
 (define-vop (raw-instance-ref/double)
   (:translate %raw-instance-ref/double)
@@ -639,7 +640,7 @@
   (:result-types double-float)
   (:generator 5
     (with-empty-tn@fp-top(value)
-      (inst fldd (make-ea-for-raw-slot object index)))))
+      (inst fldd (instance-slot-ea object index)))))
 
 (define-vop (raw-instance-set/double)
   (:translate %raw-instance-set/double)
@@ -653,7 +654,7 @@
   (:generator 5
     (unless (zerop (tn-offset value))
       (inst fxch value))
-    (inst fstd (make-ea-for-raw-slot object index))
+    (inst fstd (instance-slot-ea object index))
     (cond
       ((zerop (tn-offset value))
         (unless (zerop (tn-offset result))
@@ -672,7 +673,7 @@
   (:info index)
   (:generator 5
     (with-tn@fp-top (value)
-      (inst fstd (make-ea-for-raw-slot object index)))))
+      (inst fstd (instance-slot-ea object index)))))
 
 (define-vop (raw-instance-ref/complex-single)
   (:translate %raw-instance-ref/complex-single)
@@ -685,10 +686,10 @@
   (:generator 5
     (let ((real-tn (complex-single-reg-real-tn value)))
       (with-empty-tn@fp-top (real-tn)
-        (inst fld (make-ea-for-raw-slot object index))))
+        (inst fld (instance-slot-ea object index))))
     (let ((imag-tn (complex-single-reg-imag-tn value)))
       (with-empty-tn@fp-top (imag-tn)
-        (inst fld (make-ea-for-raw-slot object index 1))))))
+        (inst fld (instance-slot-ea object index 1))))))
 
 (define-vop (raw-instance-set/complex-single)
   (:translate %raw-instance-set/complex-single)
@@ -704,14 +705,14 @@
           (result-real (complex-single-reg-real-tn result)))
       (cond ((zerop (tn-offset value-real))
              ;; Value is in ST0.
-             (inst fst (make-ea-for-raw-slot object index))
+             (inst fst (instance-slot-ea object index))
              (unless (zerop (tn-offset result-real))
                ;; Value is in ST0 but not result.
                (inst fst result-real)))
             (t
              ;; Value is not in ST0.
              (inst fxch value-real)
-             (inst fst (make-ea-for-raw-slot object index))
+             (inst fst (instance-slot-ea object index))
              (cond ((zerop (tn-offset result-real))
                     ;; The result is in ST0.
                     (inst fst value-real))
@@ -723,7 +724,7 @@
     (let ((value-imag (complex-single-reg-imag-tn value))
           (result-imag (complex-single-reg-imag-tn result)))
       (inst fxch value-imag)
-      (inst fst (make-ea-for-raw-slot object index 1))
+      (inst fst (instance-slot-ea object index 1))
       (unless (location= value-imag result-imag)
         (inst fst result-imag))
       (inst fxch value-imag))))
@@ -736,10 +737,10 @@
   (:generator 5
     (let ((value-real (complex-single-reg-real-tn value)))
       (with-tn@fp-top (value-real)
-        (inst fst (make-ea-for-raw-slot object index))))
+        (inst fst (instance-slot-ea object index))))
     (let ((value-imag (complex-single-reg-imag-tn value)))
       (with-tn@fp-top (value-imag)
-        (inst fst (make-ea-for-raw-slot object index 1))))))
+        (inst fst (instance-slot-ea object index 1))))))
 
 (define-vop (raw-instance-ref/complex-double)
   (:translate %raw-instance-ref/complex-double)
@@ -752,10 +753,10 @@
   (:generator 7
     (let ((real-tn (complex-double-reg-real-tn value)))
       (with-empty-tn@fp-top (real-tn)
-        (inst fldd (make-ea-for-raw-slot object index))))
+        (inst fldd (instance-slot-ea object index))))
     (let ((imag-tn (complex-double-reg-imag-tn value)))
       (with-empty-tn@fp-top (imag-tn)
-        (inst fldd (make-ea-for-raw-slot object index 2))))))
+        (inst fldd (instance-slot-ea object index 2))))))
 
 (define-vop (raw-instance-set/complex-double)
   (:translate %raw-instance-set/complex-double)
@@ -771,14 +772,14 @@
           (result-real (complex-double-reg-real-tn result)))
       (cond ((zerop (tn-offset value-real))
              ;; Value is in ST0.
-             (inst fstd (make-ea-for-raw-slot object index))
+             (inst fstd (instance-slot-ea object index))
              (unless (zerop (tn-offset result-real))
                ;; Value is in ST0 but not result.
                (inst fstd result-real)))
             (t
              ;; Value is not in ST0.
              (inst fxch value-real)
-             (inst fstd (make-ea-for-raw-slot object index))
+             (inst fstd (instance-slot-ea object index))
              (cond ((zerop (tn-offset result-real))
                     ;; The result is in ST0.
                     (inst fstd value-real))
@@ -790,7 +791,7 @@
     (let ((value-imag (complex-double-reg-imag-tn value))
           (result-imag (complex-double-reg-imag-tn result)))
       (inst fxch value-imag)
-      (inst fstd (make-ea-for-raw-slot object index 2))
+      (inst fstd (instance-slot-ea object index 2))
       (unless (location= value-imag result-imag)
         (inst fstd result-imag))
       (inst fxch value-imag))))
@@ -803,10 +804,10 @@
   (:generator 20
     (let ((value-real (complex-double-reg-real-tn value)))
       (with-tn@fp-top (value-real)
-        (inst fstd (make-ea-for-raw-slot object index))))
+        (inst fstd (instance-slot-ea object index))))
     (let ((value-imag (complex-double-reg-imag-tn value)))
       (with-tn@fp-top (value-imag)
-        (inst fstd (make-ea-for-raw-slot object index 2))))))
+        (inst fstd (instance-slot-ea object index 2))))))
 
 ;;;;
 
